////////////////////////////////////////////////////////////////
// jsInvaders - Copyright (c) 2004 Oliver Wilkinson
//
// This software is provided 'as-is', without any express or implied warranty. 
// In no event will the authors be held liable for any damages arising from the use
// of this software.
//
// Permission is granted to anyone to use this software for any purpose, including 
// commercial applications, and to alter it and redistribute it freely, subject 
// to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must not claim that you 
// wrote the original software. If you use this software in a product, an acknowledgment 
// in the product documentation would be appreciated but is not required.
//
// 2. Altered source versions must be plainly marked as such, and must not be 
// misrepresented as being the original software.
//
// 3. This notice may not be removed or altered from any source distribution.
//////////////////////////////////////////////////////////////////
//
// jsInvaders is released under the zlib/png license, please see
// <http://opensource.org/licenses/zlib-license.php> for more details
//
// jsInvaders is the accompanying code to the article
// "Creating moddable games with XML and Scripting"
// by Oliver Wilkinson
//
////////////////////////////////////////////////////////////////////

#ifndef __EVO_JSIFACTORY_H_INC__
#define __EVO_JSIFACTORY_H_INC__

#include "jsInvaders.h"

namespace evolutionalcouk
{
	namespace jsInvaders
	{
		///////////////////////////////////////////////////////////////////
		// Class:		jsiFactory
		///////////////////////////////////////////////////////////////////
		// Remarks:		A basic template to handle the creation and storage
		//				of game objects
		//
		// History:		17/06/2004 - Added to project
		////////////////////////////////////////////////////////////////////
		template <class T>
		class jsiFactory
		{
		public:

			T* create()
			{
				T *obj = new T;
				m_objects.push_back(obj);
				return obj;
			}

			void destroy(T *obj)
			{
				m_objects.remove(obj);
				delete obj;
			}

			size_t count()
			{
				return m_objects.size();
			}

			T* get(size_t n)
			{
				if (n > count())
					return 0;

				return m_objects[n];
			}

			void set(size_t n, T *obj)
			{
				if (n > count())
					return;

				m_objects[n] = obj;
			}

			void clear()
			{
				for (std::vector<T*>::iterator it = m_objects.begin(); it != m_objects.end(); it++)
				{
					delete (*it);					
				}
				m_objects.clear();
			}

		private:
			// A list to store the created objects
			std::vector<T *>	m_objects;
		};

	}

}


#endif