////////////////////////////////////////////////////////////////
// jsInvaders - Copyright (c) 2004 Oliver Wilkinson
//
// This software is provided 'as-is', without any express or implied warranty. 
// In no event will the authors be held liable for any damages arising from the use
// of this software.
//
// Permission is granted to anyone to use this software for any purpose, including 
// commercial applications, and to alter it and redistribute it freely, subject 
// to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must not claim that you 
// wrote the original software. If you use this software in a product, an acknowledgment 
// in the product documentation would be appreciated but is not required.
//
// 2. Altered source versions must be plainly marked as such, and must not be 
// misrepresented as being the original software.
//
// 3. This notice may not be removed or altered from any source distribution.
//////////////////////////////////////////////////////////////////
//
// jsInvaders is released under the zlib/png license, please see
// <http://opensource.org/licenses/zlib-license.php> for more details
//
// jsInvaders is the accompanying code to the article
// "Creating moddable games with XML and Scripting"
// by Oliver Wilkinson
//
////////////////////////////////////////////////////////////////////

#ifndef __EVO_JSIGAME_H_INC__
#define __EVO_JSIGAME_H_INC__

#include "jsInvaders.h"
#include "jsiFactory.h"

#include "jsiLevel.h"
#include "jsiGameData.h"

#include "render/jsiRenderWindow.h"

namespace evolutionalcouk
{
	namespace jsInvaders
	{
		///////////////////////////////////////////////////////////////////
		// Class:		jsiGame
		///////////////////////////////////////////////////////////////////
		// Remarks:		The basic game class
		//
		//
		// History:		17/06/2004 - Added to project
		//
		//
		////////////////////////////////////////////////////////////////////
		class jsiGame
		{
		public:
			jsiGame();
			virtual ~jsiGame();
			
			///////////////////////////////////////////////////////////////////
			// Method:		initialise
			///////////////////////////////////////////////////////////////////
			// Remarks:		Called when the game is initialised (once on run)
			////////////////////////////////////////////////////////////////////
			int initialise();

			///////////////////////////////////////////////////////////////////
			// Method:		update
			///////////////////////////////////////////////////////////////////
			// Remarks:		Called every frame, updates the game
			////////////////////////////////////////////////////////////////////
			int update();

			///////////////////////////////////////////////////////////////////
			// Method:		reset
			///////////////////////////////////////////////////////////////////
			// Remarks:		Resets the game to it's initial state
			////////////////////////////////////////////////////////////////////
			int reset();

			///////////////////////////////////////////////////////////////////
			// Method:		shutdown
			///////////////////////////////////////////////////////////////////
			// Remarks:		Called at the end of the game, frees and resources allocated
			////////////////////////////////////////////////////////////////////
			int shutdown();

			///////////////////////////////////////////////////////////////////
			// Method:		createLevel
			///////////////////////////////////////////////////////////////////
			// Remarks:		Creates a level and associates it with the game
			////////////////////////////////////////////////////////////////////
			jsiLevel *createLevel();

			///////////////////////////////////////////////////////////////////
			// Method:		clearLevels
			///////////////////////////////////////////////////////////////////
			// Remarks:		Clears all the levels from the game
			////////////////////////////////////////////////////////////////////
			void clearLevels();

			///////////////////////////////////////////////////////////////////
			// Method:		loadXmlLevels
			///////////////////////////////////////////////////////////////////
			// Remarks:		Loads and parses the levels from an Xml file
			////////////////////////////////////////////////////////////////////
			int loadXmlLevels(const char *filename);

			///////////////////////////////////////////////////////////////////
			// Method:		isRunning
			///////////////////////////////////////////////////////////////////
			// Remarks:		Returns the running status of the game
			////////////////////////////////////////////////////////////////////
			bool isRunning();

		protected:
			jsiFactory<jsiLevel>	m_levels;

			// Holds the current data for the game :D
			jsiGameData m_gameData;

			bool m_gameIsRunning;

		};
	}

}

#endif
