////////////////////////////////////////////////////////////////
// jsInvaders - Copyright (c) 2004 Oliver Wilkinson
//
// This software is provided 'as-is', without any express or implied warranty. 
// In no event will the authors be held liable for any damages arising from the use
// of this software.
//
// Permission is granted to anyone to use this software for any purpose, including 
// commercial applications, and to alter it and redistribute it freely, subject 
// to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must not claim that you 
// wrote the original software. If you use this software in a product, an acknowledgment 
// in the product documentation would be appreciated but is not required.
//
// 2. Altered source versions must be plainly marked as such, and must not be 
// misrepresented as being the original software.
//
// 3. This notice may not be removed or altered from any source distribution.
//////////////////////////////////////////////////////////////////
//
// jsInvaders is released under the zlib/png license, please see
// <http://opensource.org/licenses/zlib-license.php> for more details
//
// jsInvaders is the accompanying code to the article
// "Creating moddable games with XML and Scripting"
// by Oliver Wilkinson
//
////////////////////////////////////////////////////////////////////

#ifndef __EVO_JSILEVEL_H_INC__
#define __EVO_JSILEVEL_H_INC__

#include "jsInvaders.h"
#include "jsiFactory.h"
#include "jsiLevel.h"
#include "jsiAlien.h"

namespace evolutionalcouk
{
	namespace jsInvaders
	{

		///////////////////////////////////////////////////////////////////
		// Class:		jsiLevel
		///////////////////////////////////////////////////////////////////
		// Remarks:		Holds the data for the current level, aliens, player, etc
		//
		//
		// History:		16/06/2004 - Created initial class
		//
		//
		////////////////////////////////////////////////////////////////////
		class jsiLevel
		{
		public:
			jsiLevel();
			virtual ~jsiLevel();

			///////////////////////////////////////////////////////////////////
			// Method:		createAlien
			///////////////////////////////////////////////////////////////////
			// Remarks:		Creates an alien in the game
			////////////////////////////////////////////////////////////////////
			jsiAlien* createAlien();
			///////////////////////////////////////////////////////////////////
			// Method:		getAlienCount
			///////////////////////////////////////////////////////////////////
			// Remarks:		Returns the number of active aliens in the level
			////////////////////////////////////////////////////////////////////
			size_t getAlienCount();
			///////////////////////////////////////////////////////////////////
			// Method:		getAlien
			///////////////////////////////////////////////////////////////////
			// Remarks:		Gets the alien at n offset in the internal list
			////////////////////////////////////////////////////////////////////
			jsiAlien *getAlien(size_t n);
			///////////////////////////////////////////////////////////////////
			// Method:		clearAliens
			///////////////////////////////////////////////////////////////////
			// Remarks:		Clears all the aliens from the level, deleting them in turn
			////////////////////////////////////////////////////////////////////
			void clearAliens();
			///////////////////////////////////////////////////////////////////
			// Method:		parseXmlLevel
			///////////////////////////////////////////////////////////////////
			// Remarks:		Parse an xml element to populate the level with objects
			////////////////////////////////////////////////////////////////////
			int parseXmlLevel(TiXmlElement *xLevel);


		protected:
			////////////////////////////////
			// You'll notice that this has changed from 'vector' to jsiFactory,
			// which is my quick templated factory for adding/removing objects
			// think of it as a managed list, k?
			jsiFactory<jsiAlien>	m_aliens;
			
		};
		

	}


}


#endif
