////////////////////////////////////////////////////////////////
// jsInvaders - Copyright (c) 2004 Oliver Wilkinson
//
// This software is provided 'as-is', without any express or implied warranty. 
// In no event will the authors be held liable for any damages arising from the use
// of this software.
//
// Permission is granted to anyone to use this software for any purpose, including 
// commercial applications, and to alter it and redistribute it freely, subject 
// to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must not claim that you 
// wrote the original software. If you use this software in a product, an acknowledgment 
// in the product documentation would be appreciated but is not required.
//
// 2. Altered source versions must be plainly marked as such, and must not be 
// misrepresented as being the original software.
//
// 3. This notice may not be removed or altered from any source distribution.
//////////////////////////////////////////////////////////////////
//
// jsInvaders is released under the zlib/png license, please see
// <http://opensource.org/licenses/zlib-license.php> for more details
//
// jsInvaders is the accompanying code to the article
// "Creating moddable games with XML and Scripting"
// by Oliver Wilkinson
//
////////////////////////////////////////////////////////////////////
//
// Please note that this singleton class template is adapted from Richard Fine's
// excellent 'Enginuity' article series located at:
// <http://www.gamedev.net/reference/programming/features/enginuity1>
//
////////////////////////////////////////////////////////////////////

#ifndef __EVO_JSISINGLETON_H_INC__
#define __EVO_JSISINGLETON_H_INC__

namespace evolutionalcouk
{
	namespace jsInvaders
	{
		///////////////////////////////////////////////////////////////////
		// Class:		jsiSingleton
		///////////////////////////////////////////////////////////////////
		// Remarks:		A templated singleton class to ensure only a single 
		//				instance of an object occurs
		//				
		//
		// History:		18/06/2004 - Added to project
		//
		////////////////////////////////////////////////////////////////////

		template <class T>
		class jsiSingleton
		{
		public:

			~jsiSingleton()
			{
				m_singleton = 0;
			}

			static T& getSingleton()
			{
				return (*m_singleton);
			}

			static T* getSingletonPtr()
			{
				return m_singleton;
			}

		protected:
			jsiSingleton()
			{
				// use a cunning trick to get the singleton pointing to the start of the whole, rather than
				// the start of the Singleton part of the object
				//
				// Code appears represented from Richard Fine's Enginuity Series
				//
				size_t offset = (size_t)(T*)1 - (size_t)(jsiSingleton <T>*)(T*)1;
				m_singleton = (T*)( (size_t)this + offset );
			}

		private:
			static T *m_singleton;
		};

		template <typename T> T* jsiSingleton<T>::m_singleton = 0;

	}


}


#endif
