////////////////////////////////////////////////////////////////
// jsInvaders - Copyright (c) 2004 Oliver Wilkinson
//
// This software is provided 'as-is', without any express or implied warranty. 
// In no event will the authors be held liable for any damages arising from the use
// of this software.
//
// Permission is granted to anyone to use this software for any purpose, including 
// commercial applications, and to alter it and redistribute it freely, subject 
// to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must not claim that you 
// wrote the original software. If you use this software in a product, an acknowledgment 
// in the product documentation would be appreciated but is not required.
//
// 2. Altered source versions must be plainly marked as such, and must not be 
// misrepresented as being the original software.
//
// 3. This notice may not be removed or altered from any source distribution.
//////////////////////////////////////////////////////////////////
//
// jsInvaders is released under the zlib/png license, please see
// <http://opensource.org/licenses/zlib-license.php> for more details
//
// jsInvaders is the accompanying code to the article
// "Creating moddable games with XML and Scripting"
// by Oliver Wilkinson
//
////////////////////////////////////////////////////////////////////
//
// Please note that large portions of code in this subsystem is based upon 
// Jeff Molofee's (NeHe) OpenGL tutorials located at <http://nehe.gamedev.net>
//
// Thanks to Jeff for creating such tutorials and making them public, it allows
// small games like this one to be created and shared with the game development community
//
////////////////////////////////////////////////////////////////////

#ifndef __EVO_JSIRENDERWINDOW_H_INC__
#define __EVO_JSIRENDERWINDOW_H_INC__

#include "jsInvaders.h"
#include "jsiLevel.h"
#include "jsiGameData.h"

namespace evolutionalcouk
{
	namespace jsInvaders
	{
		class jsiRenderWindow : public jsiSingleton<jsiRenderWindow>
		{
		public:
			
			///////////////////////////////////////////////////////////////////
			// Method:		Constructor
			///////////////////////////////////////////////////////////////////
			// Remarks:		You must pass a shared game data structure
			////////////////////////////////////////////////////////////////////
			jsiRenderWindow(jsiGameData *gameData);
			
			///////////////////////////////////////////////////////////////////
			// Method:		createWindow
			///////////////////////////////////////////////////////////////////
			// Remarks:		Will create the window and initialise everything opengl
			//				needs to set up
			//
			////////////////////////////////////////////////////////////////////
			int createWindow(const int width, const int height, const int depth, const bool fullscreen);
			///////////////////////////////////////////////////////////////////
			// Method:		destroyWindow
			///////////////////////////////////////////////////////////////////
			// Remarks:		Closes the window and resets the settings
			//
			////////////////////////////////////////////////////////////////////
			void destroyWindow();

			///////////////////////////////////////////////////////////////////
			// Method:		renderScene
			///////////////////////////////////////////////////////////////////
			// Remarks:		Called each time the scene is to be rendered
			//				Here's where the levels for the game are actually rendered
			//				so a pointer to the level is needed
			//
			////////////////////////////////////////////////////////////////////
			bool renderScene();


			///////////////////////////////////////////////////////////////////
			// Method:		gameWndProc
			///////////////////////////////////////////////////////////////////
			// Remarks:		The 'real' windows procedure that handles messages
			//				sent to our window
			////////////////////////////////////////////////////////////////////
			LRESULT CALLBACK gameWndProc(HWND hwnd,UINT uMsg, WPARAM wParam, LPARAM lParam);
		
		private:
			///////////////////////////////////////////////////////////////////
			// Method:		initGL
			///////////////////////////////////////////////////////////////////
			// Remarks:		Called by createWindow to initialise the OGL settings
			////////////////////////////////////////////////////////////////////
			bool initGL();
			///////////////////////////////////////////////////////////////////
			// Method:		resizeGLScene
			///////////////////////////////////////////////////////////////////
			// Remarks:		Called every time the window is resized, it resets
			//				the perspective of the window to match the current size
			////////////////////////////////////////////////////////////////////
			GLvoid resizeGLScene(GLsizei width, GLsizei height);
			
			///////////////////////////////////////////////////////////////////
			// Method:		renderCurrentLevel
			///////////////////////////////////////////////////////////////////
			// Remarks:		Called when the current level is rendered, draws all
			//				objects in the level. Current level is obtained from
			//				the jsiGameData structure passed on initialisation
			////////////////////////////////////////////////////////////////////
			bool renderCurrentLevel();
			
			
			///////////////////////////////////////////////////////////////////
			// Method:		renderGLQuad
			///////////////////////////////////////////////////////////////////
			// Remarks:		Utility for rendering a GL quad from a jsiQuad structure
			//				inlined for speed
			////////////////////////////////////////////////////////////////////
			inline void renderGLQuad(const jsiQuad &q);
			///////////////////////////////////////////////////////////////////
			// Method:		renderGLVertex
			///////////////////////////////////////////////////////////////////
			// Remarks:		Utility for rendering a GL vertex from a jsiVertex class
			//				Must be called within glBegin()/glEnd() block.
			//				inlined for speed
			////////////////////////////////////////////////////////////////////
			inline void renderGLVertex(const jsiVertex &v);

		private:
			HINSTANCE m_hInst;
			HWND	m_hWnd;
			HDC		m_hDC;
			HGLRC	m_hRC;
			bool	m_fullscreen;
			
			// Shared game data passed from game
			jsiGameData *m_gameData;
			// Stores the co-ordinates for our basic quad shape used for rendering
			// our 2d objects
			jsiQuad		m_basicShape;

		};

		
		///////////////////////////////////////////////////////////////////
		// Method:		g_gameWndProc
		///////////////////////////////////////////////////////////////////
		// Remarks:		Windows message callback -> redirect to the version 
		//				in jsiRenderWindow
		////////////////////////////////////////////////////////////////////
		LRESULT CALLBACK g_gameWndProc(HWND hwnd,UINT uMsg, WPARAM wParam, LPARAM lParam);

	}


}


#endif