////////////////////////////////////////////////////////////////
// jsInvaders - Copyright (c) 2004 Oliver Wilkinson
//
// This software is provided 'as-is', without any express or implied warranty. 
// In no event will the authors be held liable for any damages arising from the use
// of this software.
//
// Permission is granted to anyone to use this software for any purpose, including 
// commercial applications, and to alter it and redistribute it freely, subject 
// to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must not claim that you 
// wrote the original software. If you use this software in a product, an acknowledgment 
// in the product documentation would be appreciated but is not required.
//
// 2. Altered source versions must be plainly marked as such, and must not be 
// misrepresented as being the original software.
//
// 3. This notice may not be removed or altered from any source distribution.
//////////////////////////////////////////////////////////////////
//
// jsInvaders is released under the zlib/png license, please see
// <http://opensource.org/licenses/zlib-license.php> for more details
//
// jsInvaders is the accompanying code to the article
// "Creating moddable games with XML and Scripting"
// by Oliver Wilkinson
//
////////////////////////////////////////////////////////////////////

#include "jsiLevel.h"


namespace evolutionalcouk
{
	namespace jsInvaders
	{
		
		jsiLevel::jsiLevel()
		{
		}

		jsiLevel::~jsiLevel()
		{
			m_aliens.clear();
		}
		
		///////////////////////////////////////////////////////////////////
		// Method:		createAlien
		///////////////////////////////////////////////////////////////////
		// Access:		public
		// Prototype:	jsiAlien* createAlien()
		//
		// Remarks:		Creates an alien and adds it to the level
		//
		//
		// Parameters:	None
		//
		// Returns:		A pointer to the new alien
		//
		// History:		17/06/2004 - Created initial prototype
		//
		//
		////////////////////////////////////////////////////////////////////
		jsiAlien* jsiLevel::createAlien()
		{
			return m_aliens.create();
		}

		size_t jsiLevel::getAlienCount()
		{
			return m_aliens.count();
		}

		jsiAlien *jsiLevel::getAlien(size_t n)
		{
			return m_aliens.get(n);
		}

		///////////////////////////////////////////////////////////////////
		// Method:		clearAliens
		///////////////////////////////////////////////////////////////////
		// Access:		public
		// Prototype:	void clearAliens()
		//
		// Remarks:		Clears all the aliens from the level, deleting them in turn
		//
		//
		// Parameters:	None
		//
		// Returns:		None
		//
		// History:		17/06/2004 - Created initial prototype
		//				17/06/2004 - Ripped out function to replace with a generic
		//				templated factory class
		//
		////////////////////////////////////////////////////////////////////
		void jsiLevel::clearAliens()
		{
			m_aliens.clear();
		}

		///////////////////////////////////////////////////////////////////
		// Method:		parseXmlLevel
		///////////////////////////////////////////////////////////////////
		// Access:		private
		// Prototype:	int parseXmlLevel(TiXmlElement *pLevel)
		//
		// Remarks:		Parses the Xml level element to populate it with our
		//				aliens and level content
		//
		//
		// Parameters:	A pointer to the level element found
		//
		// Returns:		>0 = Success
		//
		// History:		17/06/2004 - Created
		//				05/08/2004 - Fixed bug, if no colour value specified the game will crash.
		//							 Now defaults to 'red'
		//				08/08/2004 - Added retreival of points value
		//
		////////////////////////////////////////////////////////////////////
		int jsiLevel::parseXmlLevel(TiXmlElement *xLevel)
		{
			// Here's where we should parse the file for the aliens
			TiXmlElement *xAlien = 0;
			int nAlienCount = 0;
			
			xAlien = xLevel->FirstChildElement("alien");
			// Loop through each alien in the level
			while(xAlien)
			{
				// create a new alien
				jsiAlien *alien = createAlien();
				// Parse this particular alien

				// grab the position data from the attributes
				// If there's no X/Y position in the XML document, the values default to 0
				// See: [jsiAlien.h]
				//
				// TODO: We could put this into a loop too, maybe?
				if (xAlien->Attribute("xpos"))
				{
					alien->position.x = (float)atof(xAlien->Attribute("xpos"));
				}
	
				if (xAlien->Attribute("ypos"))
				{
					alien->position.y = (float)atof(xAlien->Attribute("ypos"));
				}
				// grab the points value, if none is specified the alien carries no points
				if (xAlien->Attribute("points"))
				{
					alien->pointsValue = (size_t)atoi(xAlien->Attribute("points"));
				}

				// use my utility to return an id constant for the specified name
				// this will change when textures are implemented
				if (xAlien->Attribute("color"))
				{
					alien->color = jsiUtility::colourFromText( xAlien->Attribute("color") );
				}
				else
				{
					// no colour specified, default to red
					alien->color = jsiUtility::colourFromText( "red" );
				}

				// move to the next element
				xAlien = xAlien->NextSiblingElement("alien");
				// increment our counter
				++nAlienCount;
			}

			return 1;
		}

	}
}
