////////////////////////////////////////////////////////////////
// jsInvaders - Copyright (c) 2004 Oliver Wilkinson
//
// This software is provided 'as-is', without any express or implied warranty. 
// In no event will the authors be held liable for any damages arising from the use
// of this software.
//
// Permission is granted to anyone to use this software for any purpose, including 
// commercial applications, and to alter it and redistribute it freely, subject 
// to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must not claim that you 
// wrote the original software. If you use this software in a product, an acknowledgment 
// in the product documentation would be appreciated but is not required.
//
// 2. Altered source versions must be plainly marked as such, and must not be 
// misrepresented as being the original software.
//
// 3. This notice may not be removed or altered from any source distribution.
//////////////////////////////////////////////////////////////////
//
// jsInvaders is released under the zlib/png license, please see
// <http://opensource.org/licenses/zlib-license.php> for more details
//
// jsInvaders is the accompanying code to the article
// "Creating moddable games with XML and Scripting"
// by Oliver Wilkinson
//
////////////////////////////////////////////////////////////////////

#include "jsiLog.h"

namespace evolutionalcouk
{
	namespace jsInvaders
	{
		jsiLog::jsiLog()
		{
		}

		jsiLog::~jsiLog()
		{
		}
		///////////////////////////////////////////////////////////////////
		// Method:		createLog
		///////////////////////////////////////////////////////////////////
		// Access:		public
		// Prototype:	bool createLog(const char *filename)
		//
		// Remarks:		Creates a new logfile on disk, clears the file writes the header
		//				*** NOTE: Any existing file will be erased! ***
		//
		// Parameters:	Name/path of file to create
		//
		// Returns:		True if created, false if log couldn't be created
		//
		// History:		18/6/2004 - Created
		//
		//
		////////////////////////////////////////////////////////////////////
		bool jsiLog::createLog(const char *filename)
		{
			m_filename = filename;
			
			m_fileOut.open( m_filename, std::ios_base::out );

			if (m_fileOut.is_open() == false)
			{
				return false;
			}
			
			m_fileOut.clear();

			std::string szText = "jsInvaders - Log file\n" \
						   "---------------------\n";

			m_fileOut.write( szText.c_str(), (std::streamsize)szText.length() );

			closeLog();

			return true;
		}
		///////////////////////////////////////////////////////////////////
		// Method:		openLog
		///////////////////////////////////////////////////////////////////
		// Access:		public
		// Prototype:	bool openLog()
		//
		// Remarks:		Opens the logfile
		//
		//
		// Parameters:	None
		//
		// Returns:		True if opened, false if log couldn't be opened
		//
		// History:		18/6/2004 - Created
		//
		//
		////////////////////////////////////////////////////////////////////
		bool jsiLog::openLog()
		{
			m_fileOut.open( m_filename, std::ios_base::out | std::ios_base::app );

			if (m_fileOut.is_open() == false)
			{
				return false;
			}

			return true;
		}
		///////////////////////////////////////////////////////////////////
		// Method:		closeLog
		///////////////////////////////////////////////////////////////////
		// Access:		public
		// Prototype:	bool closeLog
		//
		// Remarks:		Closes the logfile
		//
		//
		// Parameters:	None
		//
		// Returns:		Always true
		//
		// History:		18/6/2004 - Created
		//
		//
		////////////////////////////////////////////////////////////////////
		bool jsiLog::closeLog()
		{
			m_fileOut.close();
			return true;
		}
		
		///////////////////////////////////////////////////////////////////
		// Method:		writeWarning
		///////////////////////////////////////////////////////////////////
		// Access:		public
		// Prototype:	bool writeWarning(const char *text)
		//
		// Remarks:		Will write the text to the log file with the 'WARNING' flag
		//
		//
		// Parameters:	Character string
		//
		// Returns:		True if write occurred, false if log couldn't be opened
		//
		// History:		18/6/2004 - Created
		//
		//
		////////////////////////////////////////////////////////////////////
		bool jsiLog::writeWarning(const char *text)
		{
			std::string err("WARNING:\t");
			err.append( text );
			err.append( "\n" );
			if (!openLog())
			{
				return false;
			}
			m_fileOut.write( err.c_str(), (std::streamsize)err.length() );
			closeLog();
			return true;
		}
		///////////////////////////////////////////////////////////////////
		// Method:		writeError
		///////////////////////////////////////////////////////////////////
		// Access:		public
		// Prototype:	bool writeError(const char *text)
		//
		// Remarks:		Will write the text to the log file with the 'ERROR' flag
		//
		//
		// Parameters:	Character string
		//
		// Returns:		True if write occurred, false if log couldn't be opened
		//
		// History:		18/6/2004 - Created
		//
		//
		////////////////////////////////////////////////////////////////////
		bool jsiLog::writeError(const char *text)
		{
			std::string err("ERROR:\t\t");
			err.append( text );
			err.append( "\n" );
			if (!openLog())
			{
				return false;
			}
			m_fileOut.write( err.c_str(), (std::streamsize)err.length() );
			closeLog();
			return true;
		}

	}


}