////////////////////////////////////////////////////////////////
// jsInvaders - Copyright (c) 2004 Oliver Wilkinson
//
// This software is provided 'as-is', without any express or implied warranty. 
// In no event will the authors be held liable for any damages arising from the use
// of this software.
//
// Permission is granted to anyone to use this software for any purpose, including 
// commercial applications, and to alter it and redistribute it freely, subject 
// to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must not claim that you 
// wrote the original software. If you use this software in a product, an acknowledgment 
// in the product documentation would be appreciated but is not required.
//
// 2. Altered source versions must be plainly marked as such, and must not be 
// misrepresented as being the original software.
//
// 3. This notice may not be removed or altered from any source distribution.
//////////////////////////////////////////////////////////////////
//
// jsInvaders is released under the zlib/png license, please see
// <http://opensource.org/licenses/zlib-license.php> for more details
//
// jsInvaders is the accompanying code to the article
// "Creating moddable games with XML and Scripting"
// by Oliver Wilkinson
//
////////////////////////////////////////////////////////////////////


#include "jsInvaders.h"
#include "jsiGame.h"

using namespace evolutionalcouk::jsInvaders;

int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPTSTR    lpCmdLine,
                     int       nCmdShow)

{
	////////////////////////
	// create a logfile
	jsiLog *log = new jsiLog();
	log->createLog("jsInvadersLog.txt");

	////////////////////////
	// create an instance of our game
	jsiGame jsInvadersGame;
	// initialise it
	if (!jsInvadersGame.initialise())
	{
		jsiLog::getSingleton().writeError( "main::initialise() - Failed to initialise game" );
		return 0;
	}
	//
	// Enter into a windows message loop
	// that will update the game each time it can
	//
	MSG msg;
	char bRunning = 1;

	while(bRunning)
	{
		if ( PeekMessage( &msg, NULL, 0, 0, PM_REMOVE) )
		{
			if (msg.message == WM_QUIT)
			{
				// Windows has told us to die
				bRunning = 0;
			}
			else
			{
				// Handle the windows message
				TranslateMessage(&msg);
				DispatchMessage(&msg);
			}
		}
		else
		{
			// No messages, get on with our gameloop :D
			if (jsInvadersGame.isRunning())
			{
				if (!jsInvadersGame.update())
				{
					bRunning = 0;
				}
			}
			else
			{
				bRunning = 0;
			}
		}
	}

	// All done, close down the game
	jsInvadersGame.shutdown();
	
	return 0;
}
